/*
 *    Copyright (C) 2011 by Sylvain Garsault <sylvain.garsault@gmail.com>    
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

Importer.include("util.js");
Importer.include("fileio.js");
Importer.include("amarokControl.js");
Importer.include("player.js");
Importer.include("collection.js");

/*
 * This class holds an HTML parser
 */
function HTMLParser(socket, file, arguments)
{
  // parent TCP socket
  this.socket = socket;
  // parent TCP server
  this.server = this.socket.server;
  // URL (file) to serve
  this.file = new QFileInfo(file).baseName();
  // Arguments associated to this file
  this.arguments = arguments;
  // display session
  this.session = null;
  
  // methods
  this.parse = parseHTML;
  this.parseImg = parseImg;
  this.parseArguments = parseArguments;
  this.addressCmd = addressCmd
  this.isInArgs = isInArguments;
  this.findSession = findSession;
  
  // Player
  this.updatePlaylist = updatePlaylist;
  this.buildPlayer = buildPlayer;
  this.buildPlaylist = buildPlaylist;
  
  // Remote control
  this.remoteCommands = buildRemoteCommands;
  this.parseCommand = parseCommand;
  this.parseRemote = parseRemote;
  
  // Collection
  this.buildCollection = buildCollection;
  this.displayCollection = displayCollection;
  this.displayFrom = displayFrom;
  this.displayArtist = displayArtist;
  this.displayAlbum = displayAlbum;
  this.displayTrack = displayTrack;
  this.loadItem = loadItem; 
}

/*
 * Serves HTML files
 * FIXME: only player has been implemented. Missing are collection, config and exit
 */
function parseHTML()
{
  var response = "";
  
  // Determine if there is an HTML session
  this.session = this.findSession();
  
  // Executes remote control commands if any
  if (arguments)
    this.parseCommand();
  
  d("Serving file '" + this.file +"'");
  
  var div = readFile("header.html");
  
  response += div;
  
  // Switches between the player, collection and config files
  switch(this.file)
  {
    case "":
    case "settings":
    case "player":
      response += this.buildPlayer();
    break;
    case "collection":
      response += this.buildCollection(arguments);
    break;
    case "config":
      //response += buildConfig(arguments);
    break;
    default:
      w("Unexpected HTML file to serve: " + this.file);
  }
 
   // Display something on the Freebox LED display
  response += this.session.fbxDisplayCmd();
 
  // FIXME; is it necessary to know where we come from?
  // Add remote control commands
  response += this.remoteCommands(this.addressCmd());
  
  var footer = new Template();
  footer.readFile("footer.html");
  footer.tag("session", this.session.cmd());
  response += footer;
  
  // Update session data after display
  this.session.postDisplay();
  
  return response;
}

/*
 * `Says if an attribute is in the arguments of t"<a href=\"\">Prev</a>";he HTMLParser. Returns 0 if attribute cannot be found, 
 *  else returns the index at which the first occurence has been found.
 */
function isInArguments(attribute)
{
  var result = -1;
  d("Looking for " + attribute + " in arguments[" + this.arguments.length + "]");
  for (var i=0; i<this.arguments.length; ++i)
  {
    if (this.arguments[i].attribute == attribute)
      result = i;
  }
  return (result+1);
}

/*
 * Returns a clean URL of the current page. Actions are dropped (no duplication of actions)
 */
function addressCmd()
{
  var cmd="";
  cmd = this.file+".html?";
  // Iterate over arguments, drop actions
  for (var i = 0; i< this.arguments.length; i++)
  {
    if (this.arguments[i].attribute != "action")
    cmd+= this.arguments[i].attribute+"="+this.arguments[i].value+";";
  }
  return cmd;
}

/*
 * Serves images
 * FIXME: does not work with audio CD...
 */
function parseImg(img)
{
  var response = new QByteArray();
  
  // Playlist image cover
  switch(img)
  {
    case "playListAlbumCover.png":
      var t;
      if (isPlaying())
      {
	t = Amarok.Engine.currentTrack();
      }
      else
      {
	var index=Amarok.Playlist.activeIndex();
	// Check if there is an active index
	if ((index<0))
	{
	  index=0;
	}
	if (Amarok.Playlist.totalTrackCount()>0)
	{
	  t = Amarok.Playlist.trackAt(index);
	}
      }
      if (t && t.imagePixmap())
	response = pixmapToPNG(t.imagePixmap(), 200);
      else
	response = defaultAlbumCover(200);
      break;
    
    case "albumCover.png":
      var a=0;
      var size=200;
      var b=0;
	  if (b=this.isInArgs("size"))
	    size = this.arguments[b-1].value;
	  
      if (a=this.isInArgs("id"))
      {
	var id = this.arguments[a-1].value;
	if (id != "")
	{
	  var imagePath="";
	  var imageQuery = Amarok.Collection.query("SELECT path FROM images WHERE id = "+id+";");
	  if (imageQuery.length)
	  {
	    imagePath = imageQuery[0];
	    var fi = new QFileInfo(imagePath);
	    if (fi.exists())
	    {
	      var pix = new QPixmap(imagePath, "", Qt.AutoColor);
	      if (pix != null)
		response = pixmapToPNG(pix, size);
	      else
		w("Could not load cover image id "+id+" path: '"+imagePath+"'");
	    }
	    else
	      w("Wrong image path for image id "+id+" path: '"+imagePath+"'");
	  }
	  else
	  {
	    w("Could not find cover image id " + id);
	  }
	}
	else
	{
	  w("Blank image id in URL for cover image ", id);
	}
      }
      else
	w("Missing image id in URL for cover image");
      
      //Default album cover
      if (response.length()==0)
	response = defaultAlbumCover(size);
      break;
    
    // Load image from disk
    default:
      var b=0;
      if (b=this.isInArgs("size"))
	response = readImg(img, this.arguments[b-1].value);
      else
	response = readImg(img);
      break;
  }
   
  return response;
}

/*
 * Returns the default album cover
 */
function defaultAlbumCover(size)
{
  return readImg("album.png", size);
}

/*
 * Determines which is the current HTML session, creates a new one if needed.
 */
function findSession()
{
  var arguments = this.arguments;
  var session=null;
  for (var i=0; i<arguments.length; ++i)
  {
    if (arguments[i].attribute=="session")
    {
      id = arguments[i].value;
      d("Found a session id=" + id);
      if (this.server.sessions.length >= id)
	session = this.server.sessions[id-1];
      else
	w("Session does not exist in pool");
    }
  }
  // If no session has been found, create a new one
  if (!session)
  {
    session = new Session();
    // add the session to the pool
    this.server.addSession(session);
    var peer = this.socket.socket.peerAddress();
    statusMessage("Now connected with Freebox (" + peer +")");
  }
  return session;
}

/*
 * Gets the current position in the current playing track in the format mm:ss
 */
function getCurrentPosition()
{
  return formatTime(Amarok.Engine.trackPosition());
}

/*
 * Draws a slider representing the current position in the track
 */
function buildPositionSlider(div)
{
  time = Amarok.Engine.trackPosition();
  length = (Amarok.Engine.currentTrack().length)/1000;
  // length of the slider in pixels
  sliderLength = 200;
  sliderButtonSize = 10;
  played = Math.floor((time/length)*(sliderLength-sliderButtonSize));
  toPlay = sliderLength-played;
  div.tag("posSliderPlayed", played);
  div.tag("posSliderButton", sliderButtonSize);
  div.tag("posSliderToPlay", toPlay);
}

/*
 * Clears the slider
 */
function clearPositionSlider(div)
{
  div.tag("posSliderPlayed", "0");
  div.tag("posSliderButton", "0");
  div.tag("posSliderToPlay", "0");
}

/*
 * Returns true if Amarok is playing (or in pause)
 */
ENGINE_STATE_PLAY = 0
ENGINE_STATE_PAUSE = 1

function isPlaying()
{
  var isActive;
  state = Amarok.Engine.engineState();
  switch(state)
  {
    case ENGINE_STATE_PLAY:
    case ENGINE_STATE_PAUSE:
      isActive = true;
      break;
    default:
      isActive = false;
  }
  return isActive;
}

/*
 * Refreshes the page on a specified basis
 */
function buildRefresh(time, address)
{
  return "<meta name=\"refresh\" content=\""+time+";url="+address+"\">\n";
}

/*
 * This class is a display session
 */
function Session()
{
  // Id given by the server
  this.id=0;
  // Freebox LED display
  this.fbxDisplay = new Object;
  // Selected track in the playlist
  this.selectedTrack = -1;
  // First track displayed in the playlist
  this.firstTrack = -1;
  
  // Do something when tracks change
  this.trackChanged = true;
  Amarok.Engine.trackChanged.connect(this, this.onTrackChanged);
  // FIXME treat it...
  //Amarok.Playlist.trackRemoved.connect(this, this.onTrackChanged);
}

/*
 * Returns a session command line
 */
Session.prototype.cmd = function()
{
  return QUrl.toPercentEncoding("session="+this.id+";");
}

/*
 * Update things when track changes
 */
Session.prototype.onTrackChanged = function(start, end)
{
  // Warn that a track has changed
  this.trackChanged = true;
}

/*
 * Refresh things after displaying
 */
Session.prototype.postDisplay = function()
{
  // Now that it has been displayed, it is not new anymore
  this.trackChanged = false;
  // Update LED display
  this.fbxDisplayUpdate();
}

// Length of the Freebox LED display
FBX_DSP_LEN=9;

/*
 * Loads a text on the Freebox LED display, and make it slide at @char characters per refresh, @times. If @times is 0, then it never stop displaying
 */
Session.prototype.fbxDisplayLoad = function(text, char, times)
{
  var fbx = this.fbxDisplay;
  // Convert to capital letters
  fbx.content = text.toUpperCase();
  // Replace some characters by spaces
  var regEx = new RegExp("[.',:;]", "gi");
  fbx.content = fbx.content.replace(regEx, " ");
  // Removes special characters
  regEx = new RegExp("[^-\s A-Za-z0-9_]", "gi");
  fbx.content = fbx.content.replace(regEx, "");
  // Removes empty artist, album or title fields
  regEx = new RegExp("-\\s*-","gi");
  fbx.content = fbx.content.replace(regEx, "-");
  regEx = new RegExp("^\\s*-\\s*","gi");
  fbx.content = fbx.content.replace(regEx, "");
  // Ensure that it is at least 9 chars long
  for (var i = 0; i < (FBX_DSP_LEN-fbx.content.length+1); ++i)
  {
    fbx.content += " "; 
  }
  fbx.char = char;
  fbx.start = -char;
  fbx.times = times;
  fbx.meta = "";
  this.fbxDisplayUpdate();
  d("Loaded new text for Fbx display: " + fbx.content);
}

/*
 * Updates the Freebox LED display
 */
Session.prototype.fbxDisplayUpdate = function()
{ 
  // Test if there is something to display
  if (!this.fbxDisplay.content)
  {
    w("Called fbxDisplayUpdate without any content to display.")
    return;
  }
  
  dsp = this.fbxDisplay;

  // Advance by @char characters
  dsp.start+=dsp.char;
  if (dsp.start>dsp.content.length)
  {
    dsp.start=0;
    dsp.times--;
  }
  
  // Test if the max number of repetitions has been reached
  if (dsp.times == 0)
    return;
  
  response = "<!-- Freebox display panel -->\n";
  bit = dsp.content.substring(dsp.start,Math.min(dsp.start+FBX_DSP_LEN, dsp.content.length));
  if (bit.length < FBX_DSP_LEN)
  {
    bit += dsp.content.substring(0, Math.min(FBX_DSP_LEN-bit.length, dsp.content.length));
  }
  dsp.meta =  response + "<meta name=\"panel_display\" content=\""+ bit +"\">\n";
  
}

/*
 * Returns the command to display something on the Freebox LED display
 */
Session.prototype.fbxDisplayCmd = function()
{
  response= "";
  if (this.fbxDisplay.meta)
    response = this.fbxDisplay.meta;
  return response;
}

/*
 * This class is an HTML template
 */
function Template(content)
{
  //copy content if provided
  this.content = content;
}

Template.prototype.toString = function()
{
  if (!this.content)
    w("Empty template content");
  return this.content;
}

/*
 * Reads template from a file
 */
Template.prototype.readFile = function(file)
{
  this.content = readFile(file);
}

/*
 * Replaces ###xxx### tags in the HTML templates
 */
Template.prototype.tag = function(tag, value)
{
  var regEx = new RegExp("###"+tag+"###", "gi");
  temp = this.content.replace(regEx, value);
  this.content = temp;
}
