/*
 *    Copyright (C) 2011 by Sylvain Garsault <sylvain.garsault@gmail.com>    
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Builds the collection HTML page
 */
function buildCollection()
{
  var response = "";
  var a = 0;
  
  // LED display
  this.session.fbxDisplayLoad(" COLLEC ", 0, -1);
  
   // Display a track
  if (a = this.isInArgs("track"))
    response = this.displayTrack(this.arguments[a-1].value);
  // Display an album
  else if (a = this.isInArgs("album"))
    response = this.displayAlbum(this.arguments[a-1].value);
  // Display an artist
  else if (a = this.isInArgs("artist"))
    response = this.displayArtist(this.arguments[a-1].value);
  // Display an artist or album short sub-list
  else if ((a = this.isInArgs("from")) && (b = this.isInArgs("type")))
      response = this.displayFrom(this.arguments[a-1].value, this.arguments[b-1].value);
  // Display collection
  else if (a = this.isInArgs("type"))
    response = this.displayCollection(this.arguments[a-1].value);
  // Display artist collection by default
  else
    response = this.displayCollection("artists"); 
  
  return response;
}

/*
 * Builds the collection page
 */
function displayCollection(type)
{
  var response="";
  
  // Collection template
  var div = new Template();
  div.readFile("collection.html");
  
  return response;
}

/*
 * Displays the collection for the given type = artists or albums
 */
function displayCollection(type)
{
  var response="";
  
  // Collection template
  var div = new Template();
  div.readFile("collection.html");
  
  // Stats
  div.tag("nbArtists", Amarok.Collection.totalArtists());
  div.tag("nbAlbums", Amarok.Collection.totalAlbums());
  div.tag("nbTracks", Amarok.Collection.totalTracks());
  
  list = "<table border=0 align=\"center\" valign=\"middle\">\n"
  var query = Amarok.Collection.query("SELECT UPPER(name), id FROM "+type+" ORDER BY UPPER(name);");
  var firstLetter = "";
  var letters = new Array();
  var ids = new Array();
  var othersId = 0;
  var othersReached = false;
  var re = new RegExp("[A-Z]");
  for(var i=0; (i<query.length-1) && !othersReached; i+=2)
  {
    var item = query[i];
    var id = query[i+1];
    // Test is first letter has changed
    if (item.length > 0 && item[0]!=firstLetter)
    {
      // Latin alphabet
      if (re.test(item[0]))
      {
	firstLetter = item[0];
	letters.push(firstLetter);
	ids.push(id);
	d("Found new "+type+" beginning with letter[" + letters.length + "] " + firstLetter + ": " + item + " (" + id + ")");
      }
      // Other symbols and letters
      else if (letters.length>0)
      {
	othersId = id;
	othersReached = true;
      }
    }
  }
  
  // Number of rows of letters
  var rows = 8;
  var nbLetters = Math.floor(letters.length / rows);
  if (nbLetters == 0)
    rows = 0;
  if ((letters.length % rows) > 0)
    rows +=1;
  
  for (var r = 0; r < rows; r++)
  {
    firstLetter = letters[r*nbLetters];
    var lastLetter = "";
    if (((r+1)*nbLetters-1) < letters.length)
      lastLetter = letters[(r+1)*nbLetters-1];
    else
      lastLetter = letters[letters.length-1];
    
    var focused = "";
    if (r==0)
      focused= "focused";
    
    list += "<tr><td align=\"center\"><font size=4><a href=\"collection.html?"+this.session.cmd()+"type="+type+";from="+ids[r*nbLetters]+"\" "+focused+">";
    
    if (firstLetter != lastLetter)
      list += firstLetter + "-" + lastLetter;
    else
      list += firstLetter;
    list += "</a></font></td></tr>";
  }
  
  list += "<tr><td align=\"center\"><font size=4><a href=\"collection.html?"+this.session.cmd()+"type="+type+";from="+othersId+"\">Other</a></font></td></tr>";
  
  list += "</table>\n";
  
  switch(type)
  {
    case "artists":
      div.tag("artistsBgColor", "bgcolor=\"#43ACE83F\"");
      div.tag("albumsBgColor", "bgcolor=\"#B4B4B43F\"");
      div.tag("artistsLink", "href=\"collection.html?"+this.session.cmd()+"type=artists;\"");
      div.tag("albumsLink", "onFocus=\"collection.html?"+this.session.cmd()+"type=albums;\"");
      break;
    case "albums":
      div.tag("albumsBgColor", "bgcolor=\"#43ACE83F\"");
      div.tag("artistsBgColor", "bgcolor=\"#B4B4B43F\"");
      div.tag("artistsLink", "onFocus=\"collection.html?"+this.session.cmd()+"type=artists;\"");
      div.tag("albumsLink", "hrer=\"collection.html?"+this.session.cmd()+"type=albums;\"");
      break;
    default:
      w("Unknown collection list type: " + type);
  }
  
  
  div.tag("typeName", type[0].toUpperCase()+type.substring(1));
  div.tag("type", type.substring(0, type.length-1));
  div.tag("list", list);
  
  response += div;
  
  return response;
}

/*
 * Finds a given id in a query [id, xxx], returns 0 if not found
 */
function findIdInQuery(id, query, sizeOfQuery)
{
  var i=0;
  var size = 2; // assumes a query [id, name]
  if (sizeOfQuery)
    size = sizeOfQuery;
  while((i<query.length) && (query[i]!=id))
  {
    i +=size;
  }
  if (i>=query.length)
  // id not found
  {
    if (id>=0)
      w("Id '" + id + "' not found");
    i = 0;
  }
  return i;
}

/*
 * Displays the collection from a given id
 */
function displayFrom(id, type)
{
  var response="";
  var list = "";
  var div = new Template();
  var query = Amarok.Collection.query("SELECT id, name FROM "+type+" ORDER BY UPPER(name);");
  if (query.length ==0)
  {
    list += "Nothing found<br />";
  }
  else
  {
    var i=findIdInQuery(id, query);
    d("Displaying "+type+" from id " + id + ": " + query[i]);
    var link = "collection.html?"+this.session.cmd()+"type="+type+";";
    switch(type)
    {
      case "albums":
      case "artists":
      case "tracks":
	list += displayList(query, i, link, type);
	break;
      default:
	list += displayList(query, i, link, "artists");
	break;
    }
  }
  
  
  div.readFile("itemList.html");
  div.tag("typeName", type[0].toUpperCase()+type.substring(1));
  div.tag("type", type.substring(0, type.length-1));
  // Display prev button
  div.tag("prevLink", "onFocus=\""+link+"\"");
  // Display list
  div.tag("list", list);
  
  response += div;
  return response;
}

/*
 * Displays a list of items
 */
function displayList(query, i, link, type)
{
  // Number of rows to display
  var rows = gConf.collection.itemNb;
  var upLink = "";
  var downLink = "";
  var width = "0";
  var upDown = "";
  var list="";
  
  // Display up button
  if (i>0)
  {
    upLink = "<a onFocus=\""+link+"from="+query[(Math.max(0,i-2*rows))]+";\"><table border=2 color=\"#FEFEFE3E\"><tr><td width=48><img src=\"/img/go-up.png?size=48;\" alt=\"up 1 page\"></td></tr></table></a>";
  }
  
  // Display down button
  if ((i + 2*rows) < (query.length-2))
  {
    downLink = "<a onFocus=\""+link+"from="+query[(Math.min(query.length-2,i+2*rows))]+";\"><table border=2 color=\"#FEFEFE3E\"><tr><td width=48><img src=\"/img/go-down.png?size=48;\" alt=\"up 1 page\"></td></tr></table></a>";
  }
  
  
  var nav = new Template();
  nav.readFile("upDownList.html");
  if (upLink || downLink)
  {
    upDown = "<table><tr><td height=48 width=48>"+upLink+"</td></tr><tr><td height=100 valign=\"middle\" align=\"center\"><a href=\"none\" focused><table border=2 color=\"#FEFEFE3E\"><tr><td height=24 width=24>&nbsp;</td></tr></table></a></td></tr><tr><td height=20>"+downLink+"</td></tr></table>";
    width="75";
  }
  nav.tag("upDown", upDown);
  nav.tag("width", width);
  
  
  list += "<table border=0 cellspacing=0 cellpadding=5>\n";
  for (var j = i; (j < query.length) && ((j-i) < 2*rows); j+=2)
  {
    list += "<tr><td>";
    list += "<a href=\""+link+type.substring(0,type.length-1)+"="+query[j]+";\">"+truncate(query[j+1],50)+"</a>";
    list += "</td></tr>\n";
  }
  list += "</table>\n";
  nav.tag("list", list);
  
  return nav;
}

/*
 * Displays a graphical list of albums
 */
function displayListOfAlbums(query, i, link)
{
  var response="";
  // Number of rows to display
  var rows = gConf.collection.gItemNb;
  response += "<table border=0>\n";

   // Display up button
  if (i>0)
  {
    response += "<tr><td>";
    response += "<a onFocus=\""+link+"from="+query[(Math.max(0,i-rows))]+";\"><table cellpadding=0 cellspacing=0 border=2 color=\"#FEFEFE3E\"><tr><td width=26><img src=\"/img/go-up.png?size=20;\" alt=\"up\"></td></tr></table></a>";
    response += "</td></tr>\n";
  }

  for (var j = i; (j < query.length) && ((j-i) < rows); j+=1)
  {
    var id = query[j];
    var albumQuery = Amarok.Collection.query("SELECT id, name, image FROM albums where id="+id+";");
    if (!albumQuery)
    {
      w("Could not find album id "+id);
      continue;
    }
    var trackQuery = Amarok.Collection.query("SELECT id FROM tracks where album="+id+";");
    var div = new Template();
    div.readFile("albumsList.html");
    div.tag("album", truncate(albumQuery[1], 35));
    div.tag("cover", albumCover(albumQuery[2], 75));
    var nbTracks = "";
    var nb = trackQuery.length;
    if (nb>1)
      nbTracks=nb+" tracks";
    else
      nbTracks=nb+" track";
    div.tag("nbTracks", nbTracks);
    div.tag("link", "href=\""+link+"album="+id+";\"");
    response += div;
  }
  
  // Display down button
  if (j < query.length)
  {
    response += "<tr><td>";
    response += "<a onFocus=\""+link+"from="+query[(Math.min(query.length-1,i+rows))]+";\"><table cellpadding=0 cellspacing=0 border=2 color=\"#FEFEFE3E\"><tr><td width=26><img src=\"/img/go-down.png?size=20;\" alt=\"down\"></td></tr></table></a>";
    response += "</td></tr>\n";
  }
  response += "</table>\n";
  return response;
}

/*
 * Returns the album cover URL, if it exists. Id is the image id.
 */
function albumCover(id, size)
{
  var link = "";
  if (!size)
    size=200; // default size
    if (id)
      link="<img src=\"img/albumCover.png?id="+id+";size="+size+";\">";
    else
      link="<img src=\"img/albumCover.png?size="+size+";\">";
    return link;
}


/*
 * Displays a list of tracks
 */
function displayListOfTracks(query, i, link)
{
  var response="";
  // Number of rows to display
  var rows = gConf.collection.trackNb;
  // Number of elements in the query
  var qlen = 4;
  response += "<table border=0 cellspacing=0 cellpadding=3>\n";
   // Display prev button
  if (i>0)
  {
    response += "<tr><td>";
    response += "<a href=\""+link+"from="+query[(Math.max(0,i-qlen*rows))]+";\"><table cellpadding=0 cellspacing=0 border=2 color=\"#FEFEFE3E\"><tr><td width=26><img src=\"/img/go-up.png?size=20;\" alt=\"up\"></td></tr></table></a>";
    response += "</td><td></td></tr>\n";
  }
  for (var j = i; (j < query.length) && ((j-i) < qlen*rows); j+=qlen)
  {
    var trackNumber = "";
    if (query[j+2])
      trackNumber = query[j+2]+" - ";
    var title = truncate(query[j+1],45);
    var length = formatTime(query[j+3]/1000);
    response += "<tr>";
    response += "<td><a href=\""+link+"track="+query[j]+";\">"+trackNumber +title+"</a></td><td align=\"right\" width=50>"+length+"</td>";
    response += "</tr>\n";
  }
  // Display next button
  if (j < query.length-qlen)
  {
    response += "<tr><td>";
    response += "<a href=\""+link+"from="+query[(Math.min(query.length-qlen,i+qlen*rows))]+";\"><table cellpadding=0 cellspacing=0 border=2 color=\"#FEFEFE3E\"><tr><td width=26><img src=\"/img/go-down.png?size=20;\" alt=\"down\"></td></tr></table></a>";
    response += "</td><td></td></tr>\n";
  }
  response += "</table>\n";
  return response;
}

/*
 * Displays an artist
 */
function displayArtist(id)
{
  var div = new Template();
  div.readFile("artist.html");
  
  var artistQuery = Amarok.Collection.query("SELECT id, name FROM artists where id="+id+";");
  var albumQuery = Amarok.Collection.query("SELECT id FROM albums where artist="+id+" ORDER BY name;");
  var link = "collection.html?"+this.session.cmd()+"artist="+id+";";
  var prevLink = "collection.html?"+this.session.cmd()+"type=artists;from="+id+";";
  var firstAlbum = -1;
  var a = 0;
  if (a = this.isInArgs("from"))
    firstAlbum = this.arguments[a-1].value;
  var i=findIdInQuery(firstAlbum, albumQuery,1);

  // Display prev button
  div.tag("prevLink", "onFocus=\""+prevLink+"\"");
  // Display artist name
  div.tag("artist", truncate(artistQuery[1], 50));
  // Display list of albums
  div.tag("albums", displayListOfAlbums(albumQuery,i, link));
  // Number of albums
  var nbAlbums = "";
  var nb = (albumQuery.length);
  if (nb>1)
    nbAlbums = nb+" albums";
  else
    nbAlbums = nb+" album";
  div.tag("nbAlbums", nbAlbums);

  return div;
}

/*
 * Displays an album
 */
function displayAlbum(id)
{
  var div = new Template();
  var artist = "Various artists";
  var artistId = 0;
  var album = "";
  div.readFile("album.html");

  var albumQuery = Amarok.Collection.query("SELECT id, name, artist, image FROM albums WHERE id="+id+";");
  var artistQuery = Amarok.Collection.query("SELECT id, name FROM artists WHERE id="+albumQuery[2]+";");
  var trackQuery = Amarok.Collection.query("SELECT id, title, tracknumber, length FROM tracks WHERE album="+id+" ORDER BY tracknumber;");
  album = albumQuery[1];
  if (artistQuery.length)
  {
    artist = artistQuery[1];
    artistId = artistQuery[0];
  }
  
  var link = "collection.html?"+this.session.cmd()+"album="+id+";";
  var firstTrack = -1;
  var a = 0;
  if (a = this.isInArgs("from"))
    firstTrack = this.arguments[a-1].value;
  var i=findIdInQuery(firstTrack, trackQuery, 4);
  
  var prevLink = "";
  if (a = this.isInArgs("artist")|| artistQuery.length)
    prevLink = "collection.html?"+this.session.cmd()+"artist="+artistId+";from="+id+";";
  else
    prevLink = "collection.html?"+this.session.cmd()+"type=albums;"+"from="+id+";";
  
  div.tag("album", album);
  div.tag("artist", artist);
  div.tag("cover", albumCover(albumQuery[3], 200));
  div.tag("tracks", displayListOfTracks(trackQuery,i,link));
  div.tag("loadAlbum", "href=\""+link+"action=load\"");
  div.tag("playAlbum", "href=\""+link+"action=playTrack;\"");
  div.tag("prevLink", "onFocus=\""+prevLink+"\"");
  
  var a = 0;
  var status="";
  if (a = this.isInArgs("action"))
  {
    if (this.arguments[a-1].value == "load")
    {
      status = "Added album to playlist";
    }
    if (this.arguments[a-1].value == "playTrack")
    {
      status = "Playing album...";
    }
  }
  div.tag("status", status);
  
  var len=0;
  for (var i=0; i <trackQuery.length; i+=4)
  {
    len += trackQuery[i+3]/1000;
  }
  div.tag("length", formatTime(len));
  
  var nbTracks=0;
  var nb = (trackQuery.length/4);
  if (nb>1)
    nbTracks = nb+" tracks";
  else
    nbTracks = nb+" track";
  div.tag("nbTracks", nbTracks);
  
  return div;
}

/*
 * Displays a track
 */
function displayTrack(id)
{
  var div = new Template();
  
  div.readFile("track.html");

  var trackQuery = Amarok.Collection.query("SELECT id, title, url, tracknumber, artist, album, length, composer, year FROM tracks where id="+id+";");
  var title = trackQuery[1];
  var url = trackQuery[2];
  var trackNumber = trackQuery[3];
  var artistId = trackQuery[4];
  var albumId = trackQuery[5];
  var length = Math.floor(trackQuery[6]/1000);
  var composerId = trackQuery[6];
  var yearId = trackQuery[7];
  var pathQuery = Amarok.Collection.query("SELECT rpath FROM urls WHERE id=" + url +";");
  var path = pathQuery[0];
  var artistQuery = Amarok.Collection.query("SELECT name FROM artists WHERE id=" + artistId +";");
  var artist = artistQuery[0];
  var albumQuery = Amarok.Collection.query("SELECT name, image FROM albums WHERE id=" + albumId +";");
  var album = albumQuery[0];
  var imageId = albumQuery[1];
  var composer = ""
  var composerQuery = Amarok.Collection.query("SELECT name FROM composers WHERE id=" + composerId +";");
  if (composerQuery.length)
    composer = composerQuery[0];
  var year = "";
  var yearQuery = Amarok.Collection.query("SELECT name FROM years WHERE id=" + yearId +";");
  if (yearQuery.length)
    year = yearQuery[0];
  
  var track = "";
  if (trackNumber)
    track = trackNumber+" - ";
  track += title;
    
  
  var link = this.addressCmd();
  var prevLink = "collection.html?"+this.session.cmd()+"album="+albumId+";";
  
  d("Displaying track " + title);
  response += "<a href=\""+link+"track="+id+";action=load;\">"+trackNumber +" - " + title +"</a><br />";
  
  var a = 0;  
  var loadStatus = "";
  var playStatus = "";
  if (a = this.isInArgs("action"))
  {
    if (this.arguments[a-1].value == "load")
    {
      loadStatus = "Added track to playlist";
    }
    if (this.arguments[a-1].value == "playTrack")
    {
      playStatus = "Playing track...";
    }
  }
  div.tag("loadStatus", loadStatus);
  div.tag("playStatus", playStatus);
  div.tag("cover", albumCover(imageId, 200));
  div.tag("track", track);
  div.tag("artist", artist);
  div.tag("album", album);
  div.tag("length", formatTime(length));
  div.tag("prevLink", "onFocus=\""+prevLink+"\"");
  div.tag("loadTrack", "href=\""+link+"action=load;\"");
  div.tag("playTrack", "href=\""+link+"action=playTrack;\"");
  div.tag("year", year);
  div.tag("composer", composer);
  
  return div;
}

/*
 * Loads an item in the playlist. If play is true, starts playing at the beginning of the item
 */
function loadItem(play)
{
  var jump = false;
  if (play)
    jump=true;
   // Load a track
  if (a = this.isInArgs("track"))
    response = loadTrack(this.arguments[a-1].value, jump);
  // Load all track in an album
  else if (a = this.isInArgs("album"))
    response = loadAlbum(this.arguments[a-1].value, jump);
  // Load all tracks from an artist
  else if (a = this.isInArgs("artist"))
    response = loadArtist(this.arguments[a-1].value, jump);
}


/*
 * Loads a track in the playlist
 */
function loadTrack(id, play)
{
  var trackQuery = Amarok.Collection.query("SELECT id, title, url FROM tracks where id="+id+";");
  if (!trackQuery.length)
  {
    w("Unable to load track id " + id + " into the playlist, not found in the database");
    return;
  }
  var title = trackQuery[1];
  var url = trackQuery[2];
  var pathQuery = Amarok.Collection.query("SELECT rpath, deviceid FROM urls WHERE id = " + url +";");
  var path = pathQuery[0];
  var deviceId = pathQuery[1];
  var deviceQuery = "";
  var device = "";
  if (deviceId)
  {
    deviceQuery = Amarok.Collection.query("SELECT devices.lastmountpoint FROM devices WHERE devices.id = " + deviceId + ";");
    if (deviceQuery.length)
      device = deviceQuery[0];
  }
  if (device=="")
  {
    w("Unable to find device containing the Amarok collection");
  }
  path = device + "/" + path;
  var fi = new QFileInfo(path);
  var dir = fi.canonicalPath();
  var file = fi.fileName();
  var trackUrl = new QUrl.fromLocalFile(dir+"/"+file);
  Amarok.Playlist.addMedia(trackUrl);
  d("Loaded new track in playlist: " + title);
  
  // Jump to added track
  if (play)
  {
    Amarok.Playlist.playByIndex(Amarok.Playlist.totalTrackCount()-1);
  }
}

/*
 * Loads an album in the playlist
 */
function loadAlbum(id, play)
{
  var trackQuery = Amarok.Collection.query("SELECT id FROM tracks where album="+id+" ORDER BY trackNumber;");
  var playListLength = Amarok.Playlist.totalTrackCount();
  for (var i=0; i<trackQuery.length; ++i)
  {
    loadTrack(trackQuery[i], false);
  }
  d("Loaded an album in the playlist");
  
  // Jump to first track of loaded album
  if (play && trackQuery.length)
  {
    Amarok.Playlist.playByIndex(playListLength);
  }
}