 /*
 *    Copyright (C) 2011 by Sylvain Garsault <sylvain.garsault@gmail.com>    
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 

/*
 * Builds player
 */
function buildPlayer()
{
  d("Building player...");
  
  // FIXME this should move to the generic parseCommands
  // Parse arguments
  this.parseArguments();
  
  var response = "";
  // Refresh page often
  response += buildRefresh(1, "player.html?"+this.session.cmd());
  // Read template
  var div = new Template();
  div.readFile("player.html");
  
  // Display FreeboKs version
  div.tag("version", gConf.version.major + "." + gConf.version.minor);

  // Display current track info played in Amarok
  var index = Amarok.Playlist.activeIndex();
  // Check the validity of this track
  if (index<0)
  {
    if (this.session.selectedTrack<Amarok.Playlist.totalTrackCount())
      index = this.session.selectedTrack;
    else if (Amarok.Playlist.totalTrackCount()>0)
    {
      index = 0;
    }
  }
  
  
  if( isPlaying())
  {
    var t=Amarok.Engine.currentTrack();
    div.tag("position", getCurrentPosition());
    buildPositionSlider(div);
    // Display track info on Freebox LED display
    if (this.session.trackChanged || this.isInArgs("reset"))
    {
      this.session.fbxDisplayLoad(truncate(t.title,25,"_") + " - " + truncate(t.album,20,"_") + " - " + truncate(t.artist,20,"_") + " - ", gConf.player.scrollingSpeed, -1);
    }
  }
  else
  {
    var t=Amarok.Playlist.trackAt(index);
    clearPositionSlider(div);
    div.tag("position", formatTime(0));
    // Load default message on Freebox LED display
    this.session.fbxDisplayLoad(" AMAROK ", 0, -1);
  }
  div.tag("title", truncate(t.title,50));
  div.tag("artist", "<font size=3>by </font>"+ truncate(t.artist,40));
  div.tag("album", "<font size=3>on </font>" + truncate(t.album,40));
  div.tag("length", formatTime(Math.floor(t.length/1000)));
  
  // FIXME: try to get Id from track
  //var query = Amarok.Collection.query("SELECT albums.id FROM albums, tracks WHERE tracks.id=" + t.id + " AND tracks.album = albums.id;");
  
  // Add cover image
  div.tag("cover", "playListAlbumCover.png?id="+this.server.trackTag+";");
  
  // Build playlist
  div.tag("playlist", this.buildPlaylist());
  
  response += div;
  return response;
}

/*
 * Builds the playlist
 */
function buildPlaylist()
{
  var playlist = "";
  var div = new Template();
  var selectedTrack=0;
  var firstTrack=0;
  var numTrack=8; // max that the Freebox can display
  
  // If track has just changed, select this track to display
  if (this.session.trackChanged)
    this.session.selectedTrack = -1;
  
  // Check if a given track has been selected
  if (this.session.selectedTrack >= 0)
  {
    selectedTrack=this.session.selectedTrack;
    firstTrack=this.session.firstTrack;
  }
  else
  {
      // Automatically select the current playlist track
      selectedTrack=Amarok.Playlist.activeIndex();
      // check that there was an active track...
      if (selectedTrack<0)
      {
	selectedTrack = 0;
      }
      firstTrack=Math.max(Amarok.Playlist.activeIndex()-4,0);
      this.session.firstTrack = firstTrack;
  }
  
  if (firstTrack>Amarok.Playlist.totalTrackCount()-1)
  {
    firstTrack = Math.max(0, Amarok.Playlist.totalTrackCount()-numTrack);
  }
  
  // Display scroll up symbol
  if (firstTrack>0)
    playlist+= "<tr><td><a link=\"#1A1A1A3F\" onFocus=\"player.html?"+this.session.cmd()+QUrl.toPercentEncoding("selected="+(firstTrack-1)+";\"")+"\"><table cellspacing=2 cellpadding=1 border=1 color=\"#1010103F\"><tr><td align=\"center\" height=0></td></tr></table></a></td></tr>";
  
  maxTrack = Math.min(0 + firstTrack + numTrack, Amarok.Playlist.totalTrackCount());
  
  d("Displaying playlist from " + firstTrack + " to " + maxTrack + " with selected track "+ selectedTrack );
  
  var id=0;
  for (id=firstTrack; id<maxTrack; ++id)
  {
        track = Amarok.Playlist.trackAt(id);
	div.readFile("playlist.html");
	div.tag("title", truncate(track.title,34));
	div.tag("artist", truncate(track.artist, 15));
	div.tag("album", truncate(track.album, 15));
	if (id==Amarok.Playlist.activeIndex())
	{
	  div.tag("borderColor", "color=\"#0000003F\"");
	  div.tag("backgroundColor", "bgcolor=\"#43ACE83F\"");
	}
	else
	{
	  if ((id != firstTrack) && ((id-1)!=Amarok.Playlist.activeIndex()))
	    playlist += "<tr><td cellspacing=0 cellpadding=0 border=0 align=\"center\"><box height=1 bgcolor=\"#1010103F\" width=350></td></tr>";
	  div.tag("borderColor", "");
	  div.tag("backgroundColor", "bgcolor=\"#FFFFFF3F\"");
	}
	
	// Locate the cursor on the selected track
	if (id==selectedTrack)
	{
	  div.tag("focused", "focused");
	  div.tag("link", "href=\"player.html?"+this.session.cmd()+QUrl.toPercentEncoding("jump="+id+";")+"\"");
	}
	else
	{
	  div.tag("focused", "");
	  div.tag("link", "onFocus=\"player.html?"+this.session.cmd()+QUrl.toPercentEncoding("selected="+id+";\"")+"\"");
	}
	
	// Add track to the playlist
	playlist += div;
  }
  
  // Display scroll down symbol
  if (maxTrack < Amarok.Playlist.totalTrackCount()-1)
    playlist+= "<tr><td><a link=\"#1A1A1A3F\" onFocus=\"player.html?"+this.session.cmd()+QUrl.toPercentEncoding("selected="+(maxTrack)+";\"")+"\"><table cellspacing=2 cellpadding=1 border=1 color=\"#1010103F\"><tr><td align=\"center\" height=0></td></tr></table></a></td></tr>";
  
  return playlist;
}

/*
 * `Parses arguments for the player
 */
function parseArguments()
{
  var arguments = this.arguments;
  for (var i=0; i<arguments.length; ++i)
  {
    attribute = arguments[i].attribute;
    value = arguments[i].value;
    switch(attribute)
    {
      case "selected":
	// Update first track to be displayed and track to be selected by default
	this.updatePlaylist(value);
	break;
      case "jump":
	// Jump to the selected track
	Amarok.Playlist.playByIndex(value);
	break;
      default:
	// Nothing to do
    }
  }
}

/*
 * Updates first track to be displayed and track to be selected by default
 */
function updatePlaylist(id)
{
  id = id-0 // conversion to number;
  d("User selected track " + id);
  var numTrack=8; // max that the Freebox can displayed
  var s=this.session;
  
  // Ensure that selected track exists
  if (id >= Amarok.Playlist.totalTrackCount())
  {
    s.selectedTrack = Amarok.Playlist.totalTrackCount()-1;
    w("Selected track "+ id +" disappeared");
  }
  else
  {
    s.selectedTrack = id;
  }
  
  // Ensure that first track exists
  if (s.firstTrack <0)
  {
    s.firstTrack=Math.max(s.selectedTrack-4,0);
  }
  
  // Ensure that first track is before selected track
  if (s.firstTrack>s.selectedTrack)
  {
    d("First track "+s.firstTrack+" is located after selected track "+ s.selectedTrack);
    s.firstTrack = s.selectedTrack;
  }
  
  // User trying to scroll down
  if (s.selectedTrack-s.firstTrack > numTrack-1)
  {
    s.firstTrack++;
    if (s.firstTrack>Amarok.Playlist.totalTrackCount())
      s.firstTrack = Amarok.Playlist.totalTrackCount()-1;
    d("Scrolling down to " + s.firstTrack);
  }
  d("Displaying playlist from " + s.firstTrack + " with selected track " + s.selectedTrack);
}

