 /*
 *    Copyright (C) 2011 by Sylvain Garsault <sylvain.garsault@gmail.com>    
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 Importer.loadQtBinding("qt.core");
 
 /*
 * Displays a status message in Amarok window
 */
 function statusMessage(str)
 {
   Amarok.Window.Statusbar.longMessage("<b>FreeboKs: </b>" + str);
 }
 
 /*
 * Sends debug to the console
 */
 function d(str)
 {
   if (gConf.debug.d)
   {
    dlog("I: "+str);
    Amarok.debug("I: "+str);
   }
 }
 
 /*
 * Sends warning to the console
 */
 function w(str)
 {
   if (gConf.debug.w)
   {
    dlog("W: "+str);
    Amarok.debug("W: "+str);
   }
 }
 
 /*
 * Sends errors to the console
 */
 function e(str)
 {
   dlog("E: "+str);
   if (gConf.debug.e)
    Amarok.debug("E: "+str);
 }
 
 /*
  * Opens the Log file
  */
 function openLogFile()
 {
   var path=Amarok.Info.scriptPath()+gConf.server.logDir+"/"+gConf.server.logFile;
   var file = new QFile();
   var result = openWrite(path, file);
   return file;
 }
 
 /*
  * Logs debug information in a log file
  */
 function dlog(str)
 {
   var line = "";
   line += "["+(new Date()).toUTCString()+"] ";
   line += str;
   line += "\n";
   if (gLog)
   {
      gLog.write(new QByteArray(line));
      // Flush to see the new line appear immediately in log file (empty buffer)
      gLog.flush();
   }
 }
 
 /*
 * Returns the version of the FreeboKs script
 */
function version(full)
{
  var ver = gConf.version;
  var str="";
  str += ver.major + "." + ver.minor + "." + ver.build;
  if (full)
    str += " - \"" + ver.tagline + "\"";
  return str;
}
 
 /*
 * Truncates a string, and adds "..." if it is too long to be displayed.
 */
truncate = function(str, len, trailer)
{
  if(!trailer)
    trailer = "...";
  if(str.length > len)
      return str.substring(0,(len-trailer.length))+trailer;
  else
      return str;
}

/*
 * Converts unicode to the Freebox charset (ISO 8859-15)
 *
 */
u2fbx = function(str)
{
  var tc=QTextCodec.codecForName(new QByteArray("ISO 8859-15"));
  return tc.fromUnicode(str);
}
 
 
/*
 * Formats time nicely
 */
function formatTime(s)
{
  time = "";
  minutes = Math.floor(s/60);
  seconds = s % 60;
  time = minutes+":";
  if (seconds <10)
    time += "0"+seconds;
  else
    time += seconds;
  return time;
}
 
/*
 * QString is not available within Amarok script...
 * Overloads the toString method of QByteArrays
 */
QByteArray.prototype.toString = function()
{
    ts = new QTextStream( this, QIODevice.ReadOnly );
    return ts.readAll();
}

function explode(str)
{
  var b = new QByteArray(str);
  d(b);
  for (var i=0; i<b.size(); i++)
  {
    d(b.at(i));
  }
}

/*
 * Converts a pixmap to png format, and rescales.
 */
pixmapToPNG = function(pixmap, width)
{
    data = new QByteArray();
    buffer = new QBuffer(data);
    buffer.open(QIODevice.WriteOnly);
    if (width)
      pixmap.scaledToWidth(width, Qt.SmoothTransformation).save(buffer, "PNG");
    else
      pixmap.save(buffer, "PNG");
    buffer.close();
    return data;
}